package com.mcmoddev.basemetals.init;

import com.mcmoddev.basemetals.util.Config.Options;
import com.mcmoddev.lib.blocks.BlockHumanDetector;
import com.mcmoddev.lib.material.MetalMaterial;

import net.minecraft.block.Block;

/**
 * This class initializes all blocks in Base Metals.
 *
 * @author Jasmine Iwanek
 *
 */
public class Blocks extends com.mcmoddev.lib.init.Blocks {

	public static Block human_detector;

	private static boolean initDone = false;

	protected Blocks() {
		throw new IllegalAccessError("Not a instantiable class");
	}

	/**
	 *
	 */
	public static void init() {
		if (initDone) {
			return;
		}

		Materials.init();
		ItemGroups.init();

		if (Options.enableAdamantine) {
			createBlocksFull(Materials.adamantine);
		}

		if (Options.enableAntimony) {
			createBlocksFull(Materials.antimony);
		}

		if (Options.enableAquarium) {
			createBlocksFull(Materials.aquarium);
		}

		if (Options.enableBismuth) {
			createBlocksFull(Materials.bismuth);
		}

		if (Options.enableBrass) {
			createBlocksFull(Materials.brass);
		}

		if (Options.enableBronze) {
			createBlocksFull(Materials.bronze);
		}

		if (Options.enableCharcoal) {
//			createBlock(Materials.vanilla_charcoal);
		}

		if (Options.enableCoal) {
			final MetalMaterial material = Materials.vanilla_coal;

			material.block = net.minecraft.init.Blocks.field_150402_ci;
			material.ore = net.minecraft.init.Blocks.field_150365_q;
		}

		if (Options.enableColdIron) {
			createBlocksFull(Materials.coldiron);
		}

		if (Options.enableCopper) {
			createBlocksFull(Materials.copper);
		}

		if (Options.enableCupronickel) {
			createBlocksFull(Materials.cupronickel);
		}
		
		if (Options.enableDiamond) {
			final MetalMaterial material = Materials.vanilla_diamond;

			material.block = net.minecraft.init.Blocks.field_150484_ah;
			material.ore = net.minecraft.init.Blocks.field_150482_ag;

			createBars(material);
			createDoor(material);
			createTrapDoor(material);

			createBlocksAdditional(material);
		}

		if (Options.enableElectrum) {
			createBlocksFull(Materials.electrum);
		}

		if (Options.enableGold) {
			final MetalMaterial material = Materials.vanilla_gold;

			material.block = net.minecraft.init.Blocks.field_150340_R;
			material.ore = net.minecraft.init.Blocks.field_150352_o;
			material.pressure_plate = net.minecraft.init.Blocks.field_150445_bS;

			createPlate(material);
			createBars(material);
			createDoor(material);
			createTrapDoor(material);

			createBlocksAdditional(material);
//			createButton(material);
//			createSlab(material);
//			createDoubleSlab(material);
//			createLever(material);
//			createStairs(material);
//			createWall(material);
		}

		if (Options.enableInvar) {
			createBlocksFull(Materials.invar);
		}

		if (Options.enableIron) {
			final MetalMaterial material = Materials.vanilla_iron;

			material.block = net.minecraft.init.Blocks.field_150339_S;
			material.ore = net.minecraft.init.Blocks.field_150366_p;
			material.bars = net.minecraft.init.Blocks.field_150411_aY;
			material.doorBlock = net.minecraft.init.Blocks.field_150454_av;
			material.trapdoor = net.minecraft.init.Blocks.field_180400_cw;
			material.pressure_plate = net.minecraft.init.Blocks.field_150443_bT;

			createPlate(material);

			createBlocksAdditional(material);
//			createButton(material);
//			createSlab(material);
//			createDoubleSlab(material);
//			createLever(material);
//			createStairs(material);
//			createWall(material);
		}

		if (Options.enableLead) {
			createBlocksFull(Materials.lead);
		}

		if (Options.enableMercury) {
			createOre(Materials.mercury);
			//TODO: Use this info in the Material
			//Materials.mercury.ore.setHardness(3.0f).setResistance(5.0f);
		}

		if (Options.enableMithril) {
			createBlocksFull(Materials.mithril);
		}

		if (Options.enableNickel) {
			createBlocksFull(Materials.nickel);
		}

		if (Options.enablePewter) {
			createBlocksFull(Materials.pewter);
		}

		if (Options.enablePlatinum) {
			createBlocksFull(Materials.platinum);
		}

		if (Options.enableSilver) {
			createBlocksFull(Materials.silver);
		}

		if (Options.enableStarSteel) {
			createBlocksFull(Materials.starsteel);
			Materials.starsteel.block.func_149715_a(0.5f);
			Materials.starsteel.plate.func_149715_a(0.5f);
			Materials.starsteel.ore.func_149715_a(0.5f);
			Materials.starsteel.bars.func_149715_a(0.5f);
			Materials.starsteel.doorBlock.func_149715_a(0.5f);
			Materials.starsteel.trapdoor.func_149715_a(0.5f);
		}

		if (Options.enableSteel) {
			createBlocksFull(Materials.steel);
		}

		if (Options.enableStone) {
		}

		if (Options.enableTin) {
			createBlocksFull(Materials.tin);
		}

		if (Options.enableWood) {
		}

		if (Options.enableZinc) {
			createBlocksFull(Materials.zinc);
		}

		human_detector = addBlock(new BlockHumanDetector(), "human_detector", null, ItemGroups.blocksTab);

		initDone = true;
	}
}
