package com.mcmoddev.basemetals.items;

import com.mcmoddev.basemetals.init.Materials;
import com.mcmoddev.basemetals.util.Config.Options;
import com.mcmoddev.lib.material.MetalMaterial;

import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.EnumCreatureAttribute;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.potion.Potion;
import net.minecraft.potion.PotionEffect;
import net.minecraft.util.DamageSource;
import net.minecraft.util.ResourceLocation;
import net.minecraft.world.World;

/**
 * Tool Effects
 */
public abstract class MetalToolEffects {

	private MetalToolEffects() {
		throw new IllegalAccessError("Not a instantiable class");
	}

	/**
	 *
	 * @param material The material
	 * @param item The item
	 * @param target The target
	 * @param attacker The attacker
	 */
	public static void extraEffectsOnAttack(final MetalMaterial material, final ItemStack item, final EntityLivingBase target, final EntityLivingBase attacker) {
		if (Options.enableColdIron) {
			if (material.equals(Materials.coldiron)) {
				if (target.func_70045_F()) {
					final DamageSource extraDamage = DamageSource.field_76377_j;
					target.func_70097_a(extraDamage, 3f);
				}
			}
		} else if (Options.enableAdamantine) {
			if (material.equals(Materials.adamantine)) {
				if (target.func_110138_aP() > 20f) {
					final DamageSource extraDamage = DamageSource.field_76377_j;
					target.func_70097_a(extraDamage, 4f);
				}
			}
		} else if (Options.enableMithril) {
			if (material.equals(Materials.mithril)) {
				if (target.func_70668_bt() == EnumCreatureAttribute.UNDEAD) {
					final ResourceLocation witherKey = new ResourceLocation("wither");
					final ResourceLocation blindKey = new ResourceLocation("wither");
					final PotionEffect wither = new PotionEffect(Potion.field_188414_b.func_82594_a(witherKey), 60, 3);
					final PotionEffect blind = new PotionEffect(Potion.field_188414_b.func_82594_a(blindKey), 60, 1);
					target.func_70690_d(wither);
					target.func_70690_d(blind);
				}
			}
		} else if (Options.enableAquarium) {
			if (material.equals(Materials.aquarium)) {
				if (target.func_70648_aU()) {
					final DamageSource extraDamage = DamageSource.field_76377_j;
					target.func_70097_a(extraDamage, 4f);
				}
			}
		}
	}

	/**
	 *
	 * @param material The material
	 * @param item The item
	 * @param world The world
	 * @param crafter The crafter
	 */
	public static void extraEffectsOnCrafting(final MetalMaterial material, final ItemStack item, final World world, final EntityPlayer crafter) {
		// do nothing for now. This would be where achievements or automatic enchantments could appear
	}

	/**
	 *
	 * @param material The material
	 * @param tooltipList The tooltip list
	 */
	public static void addToolSpecialPropertiesToolTip(MetalMaterial material, java.util.List<String> tooltipList) {
		if (Options.enableAdamantine) {
			if (material.equals(Materials.adamantine)) {
				tooltipList.add(net.minecraft.client.resources.I18n.func_135052_a("tooltip.adamantine.tool").replace("%x", String.valueOf(4)));
			}
		}
		if (Options.enableAquarium) {
			if (material.equals(Materials.aquarium)) {
				tooltipList.add(net.minecraft.client.resources.I18n.func_135052_a("tooltip.aquarium.tool").replace("%x", String.valueOf(4)));
			}
		}
		if (Options.enableColdIron) {
			if (material.equals(Materials.coldiron)) {
				tooltipList.add(net.minecraft.client.resources.I18n.func_135052_a("tooltip.coldiron.tool").replace("%x", String.valueOf(3)));
			}
		}
		if (Options.enableMithril) {
			if (material.equals(Materials.mithril)) {
				tooltipList.add(net.minecraft.client.resources.I18n.func_135052_a("tooltip.mithril.tool"));
			}
		}
		if (Options.enableStarSteel) {
			if (material.equals(Materials.starsteel)) {
				tooltipList.add(net.minecraft.client.resources.I18n.func_135052_a("tooltip.starsteel.tool").replace("%x", String.valueOf(10)));
			}
		}
	}

	/**
	 *
	 * @param material The material
	 * @param tooltipList The tooltip list
	 */
	public static void addArmorSpecialPropertiesToolTip(MetalMaterial material, java.util.List<String> tooltipList) {
		if (Options.enableAdamantine) {
			if (material.equals(Materials.adamantine)) {
				tooltipList.add(net.minecraft.client.resources.I18n.func_135052_a("tooltip.adamantine.armor").replace("%x", String.valueOf(4)));
			}
		}
		if (Options.enableAquarium) {
			if (material.equals(Materials.aquarium)) {
				tooltipList.add(net.minecraft.client.resources.I18n.func_135052_a("tooltip.aquarium.armor").replace("%x", String.valueOf(4)));
			}
		}
		if (Options.enableColdIron) {
			if (material.equals(Materials.coldiron)) {
				tooltipList.add(net.minecraft.client.resources.I18n.func_135052_a("tooltip.coldiron.armor").replace("%x", String.valueOf(3)));
			}
		}
		if (Options.enableMithril) {
			if (material.equals(Materials.mithril)) {
				tooltipList.add(net.minecraft.client.resources.I18n.func_135052_a("tooltip.mithril.armor"));
			}
		}
		if (Options.enableStarSteel) {
			if (material.equals(Materials.starsteel)) {
				tooltipList.add(net.minecraft.client.resources.I18n.func_135052_a("tooltip.starsteel.armor").replace("%x", String.valueOf(10)));
			}
		}
	}
}
