package com.mcmoddev.lib.blocks;

import java.util.List;

import com.mcmoddev.basemetals.registry.IOreDictionaryEntry;
import com.mcmoddev.lib.material.IMetalObject;
import com.mcmoddev.lib.material.MetalMaterial;
import com.mcmoddev.lib.util.Oredicts;

import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyDirection;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.Mirror;
import net.minecraft.util.Rotation;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

/**
 * Metal Plate
 * 
 * @author DrCyano
 *
 */
public class BlockMetalPlate extends net.minecraft.block.Block implements IOreDictionaryEntry, IMetalObject {

	/**
	 * Blockstate property
	 */
	public static final PropertyDirection FACING = PropertyDirection.func_177714_a("facing");

	private final MetalMaterial material;
	private final String oreDict;

	private static final float THICKNESS = 1.0f / 16.0f;

	private static final AxisAlignedBB[] BOXES = new AxisAlignedBB[EnumFacing.values().length];

	/**
	 *
	 * @param material The Material the plate is made from
	 */
	public BlockMetalPlate(MetalMaterial material) {
		super(Material.field_151573_f);
		this.field_149762_H = SoundType.field_185852_e;
		this.material = material;
		this.oreDict = Oredicts.PLATE + material.getCapitalizedName();
		this.field_149782_v = material.getMetalBlockHardness();
		this.field_149781_w = material.getBlastResistance();
		this.setHarvestLevel("pickaxe", material.getRequiredHarvestLevel());
		this.func_180632_j(this.field_176227_L.func_177621_b()
				.func_177226_a(FACING, EnumFacing.NORTH));
		this.field_149783_u = true;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public IBlockState func_185499_a(IBlockState state, Rotation rot) {
		return state.func_177226_a(FACING, rot.func_185831_a(state.func_177229_b(FACING)));
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public IBlockState func_185471_a(IBlockState state, Mirror mirrorIn) {
		return state.func_185907_a(mirrorIn.func_185800_a(state.func_177229_b(FACING)));
	}

	static {
		for (int i = 0; i < EnumFacing.values().length; i++) {
			final EnumFacing orientation = EnumFacing.values()[i];
			float x1 = 0;
			float x2 = 1;
			float y1 = 0;
			float y2 = 1;
			float z1 = 0;
			float z2 = 1;
			switch (orientation) {
				case DOWN:
					y1 = 1f - THICKNESS;
					break;
				case SOUTH:
					z2 = THICKNESS;
					break;
				case NORTH:
					z1 = 1f - THICKNESS;
					break;
				case EAST:
					x2 = THICKNESS;
					break;
				case WEST:
					x1 = 1f - THICKNESS;
					break;
				case UP:
				default:
					y2 = THICKNESS;
					break;
			}
			BOXES[orientation.ordinal()] = new AxisAlignedBB(x1, y1, z1, x2, y2, z2);
		}
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public boolean func_149662_c(IBlockState bs) {
		return false;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public boolean func_149686_d(IBlockState bs) {
		return false;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public IBlockState func_180642_a(final World w, final BlockPos coord, final EnumFacing face,
									 final float partialX, final float partialY, final float partialZ,
									 final int i, final EntityLivingBase placer) {
		final IBlockState defaultState = this.func_176223_P().func_177226_a(FACING, face);
		// redimension to face-local up and right dimensions
		float up;
		float right;
		EnumFacing.Axis upRotationAxis;
		EnumFacing.Axis rightRotationAxis;
		switch (face) {
			case UP: // works
				up = partialZ - 0.5F;
				right = partialX - 0.5F;
				upRotationAxis = EnumFacing.Axis.X;
				rightRotationAxis = EnumFacing.Axis.Z;
				break;
			case EAST: // works
				up = partialY - 0.5F;
				right = partialZ - 0.5F;
				upRotationAxis = EnumFacing.Axis.Z;
				rightRotationAxis = EnumFacing.Axis.Y;
				break;
			case SOUTH:
				up = 0.5F - partialY;
				right = 0.5F - partialX;
				upRotationAxis = EnumFacing.Axis.X;
				rightRotationAxis = EnumFacing.Axis.Y;
				break;
			case DOWN:
				up = 0.5F - partialZ;
				right = 0.5F - partialX;
				upRotationAxis = EnumFacing.Axis.X;
				rightRotationAxis = EnumFacing.Axis.Z;
				break;
			case WEST:
				up = 0.5F - partialY;
				right = 0.5F - partialZ;
				upRotationAxis = EnumFacing.Axis.Z;
				rightRotationAxis = EnumFacing.Axis.Y;
				break;
			case NORTH: // works
				up = partialY - 0.5F;
				right = partialX - 0.5F;
				upRotationAxis = EnumFacing.Axis.X;
				rightRotationAxis = EnumFacing.Axis.Y;
				break;
			default:
				return defaultState;
		}
		if ((Math.abs(up) < 0.25F) && (Math.abs(right) < 0.25F))
			// no rotation
			return defaultState;
		final boolean upOrRight = (up + right) > 0;
		final boolean upOrLeft = (up - right) > 0;
		if (upOrRight) {
			// up or right
			if (upOrLeft) {
				// up
				return defaultState.func_177226_a(FACING, face.func_176732_a(upRotationAxis));
			} else {
				// right
				return defaultState.func_177226_a(FACING, face.func_176732_a(rightRotationAxis).func_176734_d());
			}
		} else { 
			// down or left
			if (upOrLeft) {
				// left
				return defaultState.func_177226_a(FACING, face.func_176732_a(rightRotationAxis));
			} else {
				// down
				return defaultState.func_177226_a(FACING, face.func_176732_a(upRotationAxis).func_176734_d());
			}
		}
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public IBlockState func_176203_a(final int meta) {
		return this.func_176223_P().func_177226_a(FACING, EnumFacing.func_82600_a(meta));
	}

	@Override
	public int func_176201_c(final IBlockState bs) {
		return (bs.func_177229_b(FACING)).func_176745_a();
	}

	@Override
	protected BlockStateContainer func_180661_e() {
		return new BlockStateContainer(this, new IProperty[] { FACING });
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public AxisAlignedBB func_185496_a(final IBlockState bs, final IBlockAccess world, final BlockPos coord) {
		final EnumFacing orientation = bs.func_177229_b(FACING);
		return BOXES[orientation.ordinal()];
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public void func_185477_a(final IBlockState bs, final World world, final BlockPos coord,
									  final AxisAlignedBB box, final List<AxisAlignedBB> collisionBoxList,
									  final Entity entity) {

		final EnumFacing orientation = world.func_180495_p(coord).func_177229_b(FACING);
		super.func_185492_a(coord, box, collisionBoxList, BOXES[orientation.ordinal()]);
	}

	@Override
	public String getOreDictionaryName() {
		return this.oreDict;
	}

	@Override
	public MetalMaterial getMaterial() {
		return this.material;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public MetalMaterial getMetalMaterial() {
		return this.material;
	}
}
