package com.mcmoddev.lib.blocks;

import java.util.Random;

import com.mcmoddev.lib.material.IMetalObject;
import com.mcmoddev.lib.material.MetalMaterial;

import net.minecraft.block.BlockSlab;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.IStringSerializable;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

/**
 *
 * @author Jasmine Iwanek
 *
 */
public class BlockMetalSlab extends BlockSlab implements IMetalObject {

	public static final PropertyEnum<BlockMetalSlab.Variant> VARIANT = PropertyEnum.<BlockMetalSlab.Variant>func_177709_a("variant", BlockMetalSlab.Variant.class);

	final MetalMaterial material;

	/**
	 *
	 * @param material The material the slab is made from
	 */
	public BlockMetalSlab(MetalMaterial material) {
		super(Material.field_151573_f);
		this.func_149672_a(SoundType.field_185852_e);
		this.material = material;
		this.field_149782_v = material.getMetalBlockHardness();
		this.field_149781_w = material.getBlastResistance();
		this.setHarvestLevel("pickaxe", material.getRequiredHarvestLevel());

		IBlockState iblockstate = this.field_176227_L.func_177621_b();

		if (!this.func_176552_j())
			iblockstate = iblockstate.func_177226_a(field_176554_a, BlockSlab.EnumBlockHalf.BOTTOM);

		this.func_180632_j(iblockstate.func_177226_a(VARIANT, BlockMetalSlab.Variant.DEFAULT));
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public IBlockState func_176203_a(int meta) {
		IBlockState iblockstate = this.func_176223_P().func_177226_a(VARIANT, BlockMetalSlab.Variant.DEFAULT);

		if (!this.func_176552_j())
			iblockstate = iblockstate.func_177226_a(field_176554_a, (meta & 8) == 0 ? BlockSlab.EnumBlockHalf.BOTTOM : BlockSlab.EnumBlockHalf.TOP);

		return iblockstate;
	}

	/**
	 * Convert the BlockState into the correct metadata value
	 */
	@Override
	public int func_176201_c(IBlockState state) {
		int i = 0;

		if (!this.func_176552_j() && (state.func_177229_b(field_176554_a) == BlockSlab.EnumBlockHalf.TOP))
			i |= 8;

		return i;
	}

	@Override
	protected BlockStateContainer func_180661_e() {
		return this.func_176552_j() ? new BlockStateContainer(this, new IProperty[] { VARIANT }) : new BlockStateContainer(this, new IProperty[] { field_176554_a, VARIANT });
	}

	@Override
	public String func_150002_b(int meta) {
		return super.func_149739_a();
	}

	@Override
	public boolean func_176552_j() {
		return false;
	}

	@Override
	public IProperty<?> func_176551_l() {
		return VARIANT;
	}

	@Override
	public Comparable<?> func_185674_a(ItemStack stack) {
		return BlockMetalSlab.Variant.DEFAULT;
	}

	/**
	 * Get the Item that this Block should drop when harvested.
	 */
	@Override
	public Item func_180660_a(IBlockState state, Random rand, int fortune) {
		return this.material.slab;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public ItemStack func_185473_a(World worldIn, BlockPos pos, IBlockState state) {
		return new ItemStack(this.material.slab);
	}

	/**
	 *
	 * @author Jasmine Iwanek
	 *
	 */
	public enum Variant implements IStringSerializable {
		DEFAULT;
		@Override
		public String func_176610_l() {
			return "default";
		}
	}

	@Override
	public MetalMaterial getMaterial() {
		return this.material;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public MetalMaterial getMetalMaterial() {
		return this.material;
	}

	// @Override
	// public String getOreDictionaryName() {
	// return Oredicts.SLAB + material.getCapitalizedName();
	// }
}
