package com.mcmoddev.lib.init;

import java.lang.reflect.*;
import java.util.*;

import com.google.common.collect.*;
import com.mcmoddev.basemetals.BaseMetals;
import com.mcmoddev.basemetals.registry.IOreDictionaryEntry;
import com.mcmoddev.basemetals.util.Config.Options;
import com.mcmoddev.lib.blocks.*;
import com.mcmoddev.lib.items.*;
import com.mcmoddev.lib.material.IMetalObject;
import com.mcmoddev.lib.material.MetalMaterial;
import com.mcmoddev.lib.util.Oredicts;

import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.*;
import net.minecraftforge.fml.common.FMLLog;
import net.minecraftforge.fml.common.registry.GameRegistry;
import net.minecraftforge.oredict.OreDictionary;

/**
 * This class initializes all items in Base Metals and provides some utility
 * methods for looking up items.
 *
 * @author DrCyano
 *
 */
public abstract class Items {

	private static boolean initDone = false;

	private static BiMap<String, Item> itemRegistry = HashBiMap.create(34);
	private static Map<MetalMaterial, List<Item>> itemsByMaterial = new HashMap<>();

	private static Map<Class<?>, Integer> classSortingValues = new HashMap<>();
	private static Map<MetalMaterial, Integer> materialSortingValues = new HashMap<>();

	// public static UniversalBucket universal_bucket; // now automatically added by Forge

	protected Items() {
		throw new IllegalAccessError("Not a instantiable class");
	}

	/**
	 *
	 */
	public static void init() {
		if (initDone) {
			return;
		}

		Blocks.init();

		try {
			expandCombatArrays(net.minecraft.item.ItemAxe.class);
		}
		catch (IllegalAccessException | NoSuchFieldException ex) {
			BaseMetals.logger.error("Error modifying item classes", ex);
//			FMLLog.severe("Error modifying item classes: %s", ex);
		}

		setSortingList();
		addToMetList();
		
		initDone = true;
	}

	private static void setSortingList() {
		int ss = 0;
		classSortingValues.put(BlockMetalOre.class, ++ss * 10000);
		classSortingValues.put(BlockMetalBlock.class, ++ss * 10000);
		classSortingValues.put(BlockMetalPlate.class, ++ss * 10000);
		classSortingValues.put(BlockMetalBars.class, ++ss * 10000);
		classSortingValues.put(BlockMetalDoor.class, ++ss * 10000); // Is this needed? (ItemMetalDoor)
		classSortingValues.put(BlockMetalTrapDoor.class, ++ss * 10000);
		classSortingValues.put(InteractiveFluidBlock.class, ++ss * 10000);
		classSortingValues.put(BlockMoltenFluid.class, ++ss * 10000);
		classSortingValues.put(ItemMetalIngot.class, ++ss * 10000);
		classSortingValues.put(ItemMetalNugget.class, ++ss * 10000);
		classSortingValues.put(ItemMetalPowder.class, ++ss * 10000);
		classSortingValues.put(ItemMetalBlend.class, classSortingValues.get(ItemMetalPowder.class));
		classSortingValues.put(ItemMetalSmallPowder.class, ++ss * 10000);
		classSortingValues.put(ItemMetalSmallBlend.class, classSortingValues.get(ItemMetalSmallPowder.class));
		classSortingValues.put(ItemMetalCrackHammer.class, ++ss * 10000);
		classSortingValues.put(ItemMetalPickaxe.class, ++ss * 10000);
		classSortingValues.put(ItemMetalShovel.class, ++ss * 10000);
		classSortingValues.put(ItemMetalAxe.class, ++ss * 10000);
		classSortingValues.put(ItemMetalHoe.class, ++ss * 10000);
		classSortingValues.put(ItemMetalSword.class, ++ss * 10000);
		classSortingValues.put(ItemMetalArmor.class, ++ss * 10000);
		classSortingValues.put(ItemMetalArrow.class, ++ss * 10000);
		classSortingValues.put(ItemMetalBolt.class, ++ss * 10000);
		classSortingValues.put(ItemMetalBow.class, ++ss * 10000);
		classSortingValues.put(ItemMetalCrossbow.class, ++ss * 10000);
		classSortingValues.put(ItemMetalFishingRod.class, ++ss * 10000);
		classSortingValues.put(ItemMetalHorseArmor.class, ++ss * 10000);
		classSortingValues.put(ItemMetalShears.class, ++ss * 10000);
		classSortingValues.put(ItemMetalShield.class, ++ss * 10000);
		classSortingValues.put(ItemMetalGear.class, ++ss * 10000);
		classSortingValues.put(ItemMetalRod.class, ++ss * 10000);
		classSortingValues.put(ItemMetalDoor.class, classSortingValues.get(BlockMetalDoor.class));
		classSortingValues.put(GenericMetalItem.class, ++ss * 10000);

		classSortingValues.put(BlockButtonMetal.class, ++ss * 10000);
		classSortingValues.put(BlockMetalSlab.class, ++ss * 10000); // Is this needed? (ItemMetalSlab)
		classSortingValues.put(BlockDoubleMetalSlab.class, ++ss * 10000); // Probably not needed (ItemMetalSlab)
		classSortingValues.put(BlockHalfMetalSlab.class, ++ss * 10000); // Probably not needed (ItemMetalSlab)
		classSortingValues.put(BlockMetalLever.class, ++ss * 10000);
		classSortingValues.put(BlockMetalPressurePlate.class, ++ss * 10000);
		classSortingValues.put(BlockMetalStairs.class, ++ss * 10000); // Is this needed?
		classSortingValues.put(BlockMetalWall.class, ++ss * 10000);
		classSortingValues.put(BlockMoltenFluid.class, ++ss * 10000);
		classSortingValues.put(ItemMetalSlab.class, classSortingValues.get(BlockMetalSlab.class));
	}

	protected static void addToMetList() {
		final List<MetalMaterial> metlist = new ArrayList<>(Materials.getAllMaterials().size());
		metlist.addAll(Materials.getAllMaterials());
		metlist.sort((MetalMaterial a, MetalMaterial b) -> a.getName().compareToIgnoreCase(b.getName()));
		for (int i = 0; i < metlist.size(); i++) {
			materialSortingValues.put(metlist.get(i), i * 100);
		}
	}

	/**
	 * 
	 * @param material
	 */
	protected static void createItemsBasic(MetalMaterial material) {
		createBlend(material);
		createIngot(material);
		createNugget(material);
		createPowder(material);
		createSmallBlend(material);
		createSmallPowder(material);
	}

	/**
	 * 
	 * @param material
	 */
	protected static void createItemsFull(MetalMaterial material) {
		createArrow(material);
		createAxe(material);
		createBlend(material);
		createBolt(material);
		createBoots(material);
		createBow(material);
		createChestplate(material);
		createCrackhammer(material);
		createCrossbow(material);
		createDoor(material);
		createFishingRod(material);
		createHelmet(material);
		createHoe(material);
		createHorseArmor(material);
		createIngot(material);
		createLeggings(material);
		createNugget(material);
		createPickaxe(material);
		createPowder(material);
		createShears(material);
		createShield(material);
		createShovel(material);
		createSlab(material);
		createSmallBlend(material);
		createSmallPowder(material);
		createSword(material);
		createRod(material);
		createGear(material);
	}

	/**
	 * 
	 * @param material
	 */
	protected static void createItemsModSupport(MetalMaterial material) {
		createItemsModMekanism(material);
		createItemsModIC2(material);
	}

	/**
	 * 
	 * @param material
	 */
	protected static void createItemsModIC2(MetalMaterial material) {
		final boolean enableModderSupportThings = true;

		if (material.hasOre) {
			createCrushed(material);
			createCrushedPurified(material);
		}

		if (enableModderSupportThings) {
			createCasing(material);
			createDensePlate(material);
		}
	}

	/**
	 * 
	 * @param material
	 */
	protected static void createItemsModMekanism(MetalMaterial material) {
		if (material.hasOre) {
			createCrystal(material);
			createShard(material);
			createClump(material);
			createDirtyPowder(material);
		}
	}

	/**
	 * 
	 * @param item
	 * @param name
	 * @param material
	 * @param tab
	 * @return
	 */
	protected static Item addItem(Item item, String name, MetalMaterial material, CreativeTabs tab) {

		String fullName;
		if (material != null) {
			fullName = material.getName() + "_" + name;
		} else {
			fullName = name;
		}

		item.setRegistryName(fullName);
		item.func_77655_b(item.getRegistryName().func_110624_b() + "." + fullName);
		GameRegistry.register(item);
		itemRegistry.put(fullName, item);

		if (tab != null) {
			item.func_77637_a(tab);
		}

		if (material != null) {
			itemsByMaterial.computeIfAbsent(material, (MetalMaterial g) -> new ArrayList<>());
			itemsByMaterial.get(material).add(item);
		}

		if (item instanceof IOreDictionaryEntry) {
			OreDictionary.registerOre(((IOreDictionaryEntry) item).getOreDictionaryName(), item);
		}

		return item;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createIngot(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableBasics) && (material.ingot == null)) {
			material.ingot = addItem(new ItemMetalIngot(material), "ingot", material, ItemGroups.ItemsTab);
		}

		return material.ingot;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createNugget(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableBasics) && (material.nugget == null)) {
			material.nugget = addItem(new ItemMetalNugget(material), "nugget", material, ItemGroups.ItemsTab);
		}

		return material.nugget;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createPowder(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableBasics) && (material.powder == null)) {
			material.powder = addItem(new ItemMetalPowder(material), "powder", material, ItemGroups.ItemsTab);
		}

		return material.powder;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createBlend(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableBasics) && (material.hasBlend) && (material.blend == null)) {
			material.blend = addItem(new ItemMetalBlend(material), "blend", material, ItemGroups.ItemsTab);
		}

		return material.blend;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createRod(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableRod) && (material.rod == null)) {
			material.rod = addItem(new ItemMetalRod(material), "rod", material, ItemGroups.ItemsTab);
			OreDictionary.registerOre(Oredicts.STICK + material.getCapitalizedName(), material.rod);
			OreDictionary.registerOre(Oredicts.ROD, material.rod);
		}

		return material.rod;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createGear(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableGear) && (material.gear == null)) {
			material.gear = addItem(new ItemMetalGear(material), "gear", material, ItemGroups.ItemsTab);
			OreDictionary.registerOre(Oredicts.GEAR, material.gear);
		}

		return material.gear;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createAxe(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableBasicTools) && (material.axe == null)) {
			material.axe = addItem(new ItemMetalAxe(material), "axe", material, ItemGroups.toolsTab);
		}

		return material.axe;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createCrackhammer(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableCrackHammer) && (material.crackhammer == null)) {
			material.crackhammer = addItem(new ItemMetalCrackHammer(material), "crackhammer", material, ItemGroups.toolsTab);
		}

		return material.crackhammer;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createHoe(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableBasicTools) && (material.hoe == null)) {
			material.hoe = addItem(new ItemMetalHoe(material), "hoe", material, ItemGroups.toolsTab);
		}

		return material.hoe;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createPickaxe(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableBasicTools) && (material.pickaxe == null)) {
			material.pickaxe = addItem(new ItemMetalPickaxe(material), "pickaxe", material, ItemGroups.toolsTab);
		}

		return material.pickaxe;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createShovel(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableBasicTools) && (material.shovel == null)) {
			material.shovel = addItem(new ItemMetalShovel(material), "shovel", material, ItemGroups.toolsTab);
		}

		return material.shovel;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createSword(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableBasicTools) && (material.sword == null)) {
			material.sword = addItem(new ItemMetalSword(material), "sword", material, ItemGroups.toolsTab);
		}

		return material.sword;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createHelmet(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableArmor) && (material.helmet == null)) {
			material.helmet = addItem(ItemMetalArmor.createHelmet(material), "helmet", material, ItemGroups.toolsTab);
		}

		return material.helmet;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createChestplate(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableArmor) && (material.chestplate == null)) {
			material.chestplate = addItem(ItemMetalArmor.createChestplate(material), "chestplate", material, ItemGroups.toolsTab);
		}

		return material.chestplate;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createLeggings(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableArmor) && (material.leggings == null)) {
			material.leggings = addItem(ItemMetalArmor.createLeggings(material), "leggings", material, ItemGroups.toolsTab);
		}

		return material.leggings;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createBoots(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableArmor) && (material.boots == null)) {
			material.boots = addItem(ItemMetalArmor.createBoots(material), "boots", material, ItemGroups.toolsTab);
		}

		return material.boots;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createHorseArmor(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableHorseArmor) && (material.horse_armor == null)) {
			material.horse_armor = addItem(new ItemMetalHorseArmor(material), "horsearmor", material, ItemGroups.toolsTab);
		}

		return material.horse_armor;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createArrow(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableBowAndArrow) && (material.arrow == null)) {
			material.arrow = addItem(new ItemMetalArrow(material), "arrow", material, ItemGroups.toolsTab);
		}

		return material.arrow;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createBolt(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableCrossbowAndBolt) && (material.bolt == null)) {
			material.bolt = addItem(new ItemMetalBolt(material), "bolt", material, ItemGroups.toolsTab);
		}

		return material.bolt;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createBow(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableBowAndArrow) && (material.bow == null)) {
			material.bow = addItem(new ItemMetalBow(material), "bow", material, ItemGroups.toolsTab);
		}

		return material.bow;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createCrossbow(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableCrossbowAndBolt) && (material.crossbow == null)) {
			material.crossbow = addItem(new ItemMetalCrossbow(material), "crossbow", material, ItemGroups.toolsTab);
		}

		return material.crossbow;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createShears(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableShears) && (material.shears == null)) {
			material.shears = addItem(new ItemMetalShears(material), "shears", material, ItemGroups.toolsTab);
		}

		return material.shears;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createSmallBlend(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableSmallDust) && (material.hasBlend) && (material.smallblend == null)) {
			material.smallblend = addItem(new ItemMetalSmallBlend(material), "smallblend", material, ItemGroups.ItemsTab);
		}

		return material.smallblend;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createFishingRod(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableFishingRod) && (material.fishing_rod == null)) {
			material.fishing_rod = addItem(new ItemMetalFishingRod(material), "fishing_rod", material, ItemGroups.toolsTab);
		}

		return material.fishing_rod;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createSmallPowder(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableSmallDust) && (material.smallpowder == null)) {
			material.smallpowder = addItem(new ItemMetalSmallPowder(material), "smallpowder", material, ItemGroups.ItemsTab);
		}

		return material.smallpowder;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createShield(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableShield) && (material.shield == null)) {
			material.shield = addItem(new ItemMetalShield(material), "shield", material, ItemGroups.ItemsTab);
		}

		return material.shield;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createCrystal(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableMekanism) && (material.crystal == null)) {
			material.crystal = addItem(new GenericMetalItem(material), "crystal", material, ItemGroups.ItemsTab);
			OreDictionary.registerOre(Oredicts.CRYSTAL + material.getCapitalizedName(), material.crystal);
		}

		return material.crystal;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createShard(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableMekanism) && (material.shard == null)) {
			material.shard = addItem(new GenericMetalItem(material), "shard", material, ItemGroups.ItemsTab);
			OreDictionary.registerOre(Oredicts.SHARD + material.getCapitalizedName(), material.shard);
		}

		return material.shard;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createClump(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableMekanism) && (material.clump == null)) {
			material.clump = addItem(new GenericMetalItem(material), "clump", material, ItemGroups.ItemsTab);
			OreDictionary.registerOre(Oredicts.CLUMP + material.getCapitalizedName(), material.clump);
		}

		return material.clump;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createDirtyPowder(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableMekanism) && (material.powder_dirty == null)) {
			material.powder_dirty = addItem(new GenericMetalItem(material), "powder_dirty", material, ItemGroups.ItemsTab);
			OreDictionary.registerOre(Oredicts.DUSTDIRTY + material.getCapitalizedName(), material.powder_dirty);
		}

		return material.powder_dirty;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createCasing(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableIC2) && (material.casing == null)) {
			material.casing = addItem(new GenericMetalItem(material), "casing", material, ItemGroups.ItemsTab);
			OreDictionary.registerOre(Oredicts.CASING + material.getCapitalizedName(), material.casing);
		}

		return material.casing;
	}

	// TODO: Possibly make this a block, 1/2 of the normal plate.
	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createDensePlate(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableIC2) && (material.dense_plate == null)) {
			material.dense_plate = addItem(new GenericMetalItem(material), "dense_plate", material, ItemGroups.ItemsTab);
			OreDictionary.registerOre(Oredicts.PLATEDENSE + material.getCapitalizedName(), material.dense_plate);
		}

		return material.dense_plate;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createCrushed(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableIC2) && (material.crushed == null)) {
			material.crushed = addItem(new GenericMetalItem(material), "crushed", material, ItemGroups.ItemsTab);
			OreDictionary.registerOre(Oredicts.CRUSHED + material.getCapitalizedName(), material.crushed);
		}

		return material.crushed;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createCrushedPurified(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableIC2) && (material.crushed_purified == null)) {
			material.crushed_purified = addItem(new GenericMetalItem(material), "crushed_purified", material, ItemGroups.ItemsTab);
			OreDictionary.registerOre(Oredicts.CRUSHEDPURIFIED + material.getCapitalizedName(), material.crushed_purified);
		}

		return material.crushed_purified;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createSlab(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableSlab) && (material.slab == null)) {
			// TODO: Do we need to check for the block too?
			if ((material.half_slab != null) && (material.double_slab != null)) {
				material.slab = addItem(new ItemMetalSlab(material), "slab", material, ItemGroups.blocksTab);
			}
		}

		return material.slab;
	}

	/**
	 * 
	 * @param material
	 * @return
	 */
	protected static Item createDoor(MetalMaterial material) {
		if (material == null) {
			return null;
		}

		if ((Options.enableDoor) && (material.door == null)) {
			if (material.doorBlock != null) {
				material.door = addItem(new ItemMetalDoor(material), "door", material, ItemGroups.blocksTab);
				OreDictionary.registerOre(Oredicts.DOOR, material.door);
			}
		}

		return material.door;
	}

	/**
	 * Uses reflection to expand the size of the combat damage and attack speed
	 * arrays to prevent initialization index-out-of-bounds errors
	 *
	 * @param itemClass The class to modify
	 */
	private static void expandCombatArrays(Class<?> itemClass) throws IllegalAccessException, NoSuchFieldException {
		// WARNING: this method contains black magic
		final int expandedSize = 256;
		final Field[] fields = itemClass.getDeclaredFields();
		for (final Field f : fields) {
			if (Modifier.isStatic(f.getModifiers()) && f.getType().isArray() && f.getType().getComponentType().equals(float.class)) {
				BaseMetals.logger.info("%s: Expanding array variable %s.%s to size %d", Thread.currentThread().getStackTrace()[0].toString(), itemClass.getSimpleName(), f.getName(), expandedSize);
				           FMLLog.info("%s: Expanding array variable %s.%s to size %s", Thread.currentThread().getStackTrace()[0], itemClass.getSimpleName(), f.getName(), expandedSize);
				f.setAccessible(true); // bypass 'private' key word
				final Field modifiersField = Field.class.getDeclaredField("modifiers");
				modifiersField.setAccessible(true);
				modifiersField.setInt(f, f.getModifiers() & ~Modifier.FINAL); // bypass 'final' key word
				final float[] newArray = new float[expandedSize];
				Arrays.fill(newArray, 0F);
				System.arraycopy(f.get(null), 0, newArray, 0, Array.getLength(f.get(null)));
				f.set(null, newArray);
			}
		}
	}

	/**
	 *
	 * @param a The itemstack
	 * @return The output
	 */
	public static int getSortingValue(ItemStack a) {
		int classVal = 990000;
		int materialVal = 9900;
		if ((a.func_77973_b() instanceof ItemBlock) && (((ItemBlock) a.func_77973_b()).func_179223_d() instanceof IMetalObject)) {
			classVal = classSortingValues.computeIfAbsent(((ItemBlock) a.func_77973_b()).func_179223_d().getClass(), (Class<?> c) -> 990000);
			materialVal = materialSortingValues.computeIfAbsent(((IMetalObject) ((ItemBlock) a.func_77973_b()).func_179223_d()).getMaterial(), (MetalMaterial m) -> 9900);
		} else if (a.func_77973_b() instanceof IMetalObject) {
			classVal = classSortingValues.computeIfAbsent(a.func_77973_b().getClass(), (Class<?> c) -> 990000);
			materialVal = materialSortingValues.computeIfAbsent(((IMetalObject) a.func_77973_b()).getMaterial(), (MetalMaterial m) -> 9900);
		}
		return classVal + materialVal + (a.func_77960_j() % 100);
	}

	/**
	 * Gets an item by its name. The name is the name as it is registered in the
	 * GameRegistry, not its unlocalized name (the unlocalized name is the
	 * registered name plus the prefix "basemetals.")
	 *
	 * @param name The name of the item in question
	 * @return The item matching that name, or null if there isn't one
	 */
	public static Item getItemByName(String name) {
		return itemRegistry.get(name);
	}

	/**
	 * This is the reverse of the getItemByName(...) method, returning the
	 * registered name of an item instance (Base Metals items only).
	 *
	 * @param i The item in question
	 * @return The name of the item, or null if the item is not a Base Metals
	 * item.
	 */
	public static String getNameOfItem(Item i) {
		return itemRegistry.inverse().get(i);
	}

	public static Map<String, Item> getItemRegistry() {
		return itemRegistry;
	}

	/**
	 * Gets a map of all items added, sorted by material
	 *
	 * @return An unmodifiable map of added items catagorized by metal material
	 */
	public static Map<MetalMaterial, List<Item>> getItemsByMaterial() {
		return Collections.unmodifiableMap(itemsByMaterial);
	}
}
