package com.mcmoddev.lib.items;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.EntityEquipmentSlot;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.potion.Potion;
import net.minecraft.potion.PotionEffect;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.FMLLog;
import net.minecraftforge.fml.common.Loader;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.oredict.OreDictionary;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

import javax.annotation.Nonnull;

import com.mcmoddev.basemetals.init.Achievements;
import com.mcmoddev.basemetals.init.Materials;
import com.mcmoddev.basemetals.items.MetalToolEffects;
import com.mcmoddev.basemetals.util.Config.Options;
import com.mcmoddev.lib.material.IMetalObject;
import com.mcmoddev.lib.material.MetalMaterial;

import net.minecraft.item.ItemArmor.ArmorMaterial;

/**
 * Armor
 * @author DrCyano
 *
 */
public class ItemMetalArmor extends net.minecraft.item.ItemArmor implements IMetalObject {

    private static final ResourceLocation speedPotionKey = new ResourceLocation("speed");
    private static final ResourceLocation jumpPotionKey = new ResourceLocation("jump_boost");
    private static final ResourceLocation slowPotionKey = new ResourceLocation("slowness");
    private static final ResourceLocation protectionPotionKey = new ResourceLocation("resistance");
    private static final ResourceLocation waterBreathingPotionKey = new ResourceLocation("water_breathing");
    private static final ResourceLocation waterBuffPotionKey = new ResourceLocation("resistance");
    private static final ResourceLocation fatiguePotionKey = new ResourceLocation("mining_fatigue");
    private static final ResourceLocation fireproofPotionKey = new ResourceLocation("fire_resistance");

    private final String customTexture;
	private final MetalMaterial material;
	private final String repairOreDictName;

	private static final int UPDATE_INTERVAL = 11;
	private static final int EFFECT_DURATION = 45;
	private static final Map<EntityPlayer, AtomicLong> playerUpdateTimestampMap = new HashMap<>();
	private static final Map<EntityPlayer, AtomicInteger> playerUpdateCountMap = new HashMap<>();

	private static final Map<EntityPlayer, AtomicInteger> starsteelUpdateCache = new HashMap<>();
	private static final Map<EntityPlayer, AtomicInteger> adamantineUpdateCache = new HashMap<>();
	private static final Map<EntityPlayer, AtomicInteger> leadUpdateCache = new HashMap<>();

	protected ItemMetalArmor(@Nonnull MetalMaterial material, @Nonnull ArmorMaterial armorMat, int renderIndex,
							 EntityEquipmentSlot slot) {
		super(armorMat, renderIndex, slot);
		this.material = material;
		this.repairOreDictName = "ingot" + material.getCapitalizedName();
		this.customTexture = Loader.instance().activeModContainer().getModId() + ":textures/models/armor/" + material.getName() + "_layer_" + (slot == EntityEquipmentSlot.LEGS ? 2 : 1) + ".png";
	}

	@Override
	public void onArmorTick(World w, EntityPlayer player, ItemStack armor) {
		if(!(playerUpdateTimestampMap.containsKey(player))) {
			playerUpdateTimestampMap.put(player, new AtomicLong(0));
			playerUpdateCountMap.put(player, new AtomicInteger(0));
			return;
		}
		if(!w.field_72995_K && w.func_82737_E() > playerUpdateTimestampMap.get(player).get()) {
			playerUpdateTimestampMap.get(player).set(w.func_82737_E() + UPDATE_INTERVAL);
			int updateCount = playerUpdateCountMap.get(player).getAndIncrement();
			for(int i = 0; i < 4; i++) {
				if(player.field_71071_by.field_70460_b[i] != null && player.field_71071_by.field_70460_b[i].func_77973_b() instanceof ItemMetalArmor) {
					doArmorUpdate(w, player, player.field_71071_by.field_70460_b[i], updateCount);
				}
			}
		}
	}

	protected void doArmorUpdate(final World w, final EntityPlayer player, final ItemStack armor, int i) {
		// some sanity checks
		if(armor == null)
			return;
		if(armor.func_77973_b() == null)
			return;
		if(player == null)
			return;
		Item armorItem = armor.func_77973_b();
		if(i % 2 == 0) {
			// count armor pieces
			if ((Options.enableStarSteel) && (((ItemMetalArmor)armorItem).material.getName().equals("starsteel"))) {
				starsteel: {
					// used to count up the starsteel armor items
					if(!(starsteelUpdateCache.containsKey(player))) {
						starsteelUpdateCache.put(player, new AtomicInteger(0));
					}
					starsteelUpdateCache.get(player).incrementAndGet();
					// Achievement
                   	if (Options.enableAchievements) {
                   		if(armorItem == Materials.starsteel.boots)
                   			player.func_71064_a(Achievements.moonBoots, 1);
                   	}
					break starsteel;
				}
			}
			if ((Options.enableLead) && (((ItemMetalArmor)armorItem).material.getName().equals("lead"))) {
				lead: {
					// used to count up the starsteel armor items
					if(!(leadUpdateCache.containsKey(player))) {
						leadUpdateCache.put(player, new AtomicInteger(0));
					}
					leadUpdateCache.get(player).incrementAndGet();
					break lead;
				}
			}
			if ((Options.enableAdamantine) && (((ItemMetalArmor)armorItem).material.getName().equals("adamantine"))) {
				adamantine: {
					// used to count up the adamantine armor items
					if(!(adamantineUpdateCache.containsKey(player))) {
						adamantineUpdateCache.put(player, new AtomicInteger(0));
					}
					adamantineUpdateCache.get(player).incrementAndGet();
					break adamantine;
				}
			}
		} else {
			// apply potion effects. Note that "Level I" is actually effect level 0 in the effect constructor 
			if (Options.enableStarSteel) {
				starsteel: {
					if(!starsteelUpdateCache.containsKey(player))
						break starsteel;
					int num = starsteelUpdateCache.get(player).getAndSet(0);
					if(num == 0)
						break starsteel;
					final PotionEffect jumpBoost = new PotionEffect(Potion.field_188414_b.func_82594_a(jumpPotionKey), EFFECT_DURATION, num - 1, false, false);
					player.func_70690_d(jumpBoost);
					if(num > 1) {
						final PotionEffect speedBoost = new PotionEffect(Potion.field_188414_b.func_82594_a(speedPotionKey), EFFECT_DURATION, num - 2, false, false);
						player.func_70690_d(speedBoost);
					}
					break starsteel;
				}
			}
			if (Options.enableLead) {
				lead: {
					if(!(leadUpdateCache.containsKey(player)))
						break lead;
					int level = leadUpdateCache.get(player).getAndSet(0) / 2;
					if(level == 0)
						break lead;
					if(level > 0) {
						final PotionEffect speedLoss = new PotionEffect(Potion.field_188414_b.func_82594_a(slowPotionKey), EFFECT_DURATION, level - 1, false, false);
						player.func_70690_d(speedLoss);
					}
					break lead;
				}
			}
			if (Options.enableAdamantine) {
				adamantine: {
					if(!(adamantineUpdateCache.containsKey(player)))
						break adamantine;
					int num = adamantineUpdateCache.get(player).getAndSet(0);
					int level = num / 2;
					if(level == 0)
						break adamantine;
					if(level > 0) {
						final PotionEffect protection = new PotionEffect(Potion.field_188414_b.func_82594_a(protectionPotionKey), EFFECT_DURATION, level - 1, false, false);
						player.func_70690_d(protection);
					}
					// Achievement
                	if ((Options.enableAchievements) && (num == 4)) {
                		player.func_71064_a(Achievements.juggernaut, 1);
                	}
					break adamantine;
				}
			}
			// full suit of cold-iron makes you fire-proof
			if (Options.enableColdIron) {
				if(armorItem == Materials.coldiron.helmet) {
					if(player.field_71071_by.field_70460_b[2] != null && player.field_71071_by.field_70460_b[2].func_77973_b() == Materials.coldiron.chestplate
							&& player.field_71071_by.field_70460_b[1] != null && player.field_71071_by.field_70460_b[1].func_77973_b() == Materials.coldiron.leggings
							&& player.field_71071_by.field_70460_b[0] != null && player.field_71071_by.field_70460_b[0].func_77973_b() == Materials.coldiron.boots) {
						final PotionEffect fireProtection = new PotionEffect(Potion.field_188414_b.func_82594_a(fireproofPotionKey), EFFECT_DURATION, 0, false, false);
						player.func_70690_d(fireProtection);
						// Achievement
                    	if (Options.enableAchievements) {
                    		if(player.func_184614_ca() != null && player.func_184614_ca().func_77973_b() == Materials.coldiron.sword) {
                    			player.func_71064_a(Achievements.demonSlayer, 1);
                    		}
                    	}
					}
				}
			}
			// full suit of mithril protects you from withering, poison, nausea, and hunger effects
			if (Options.enableMithril) {
				if(armorItem == Materials.mithril.helmet) {
					if(player.field_71071_by.field_70460_b[2] != null && player.field_71071_by.field_70460_b[2].func_77973_b() == Materials.mithril.chestplate
							&& player.field_71071_by.field_70460_b[1] != null && player.field_71071_by.field_70460_b[1].func_77973_b() == Materials.mithril.leggings
							&& player.field_71071_by.field_70460_b[0] != null && player.field_71071_by.field_70460_b[0].func_77973_b() == Materials.mithril.boots) {
                    	final List<Potion> removeList = new LinkedList<>(); // needed to avoid concurrent modification error
                    	Iterator<PotionEffect> effectIterator = player.func_70651_bq().iterator();
                    	while(effectIterator.hasNext()) {
                        	PotionEffect pe = effectIterator.next();
                        	Potion p = pe.func_188419_a();
                        	if(p.func_76398_f()) {
                        		removeList.add(p);
                        	}
                    	}
                    	for(Potion p : removeList) {
                    		player.func_184589_d(p);
                    	}
                    	// Achievement
                    	if (Options.enableAchievements) {
                    		if(player.func_184614_ca() != null && player.func_184614_ca().func_77973_b() == Materials.mithril.sword) {
								player.func_71064_a(Achievements.angelOfDeath, 1);
							}
                    	}
					}
				}
			}
			// full suit of aquarium makes you breathe and heal underwater
			if (Options.enableAquarium) {
				if(armorItem == Materials.aquarium.helmet && player.field_70163_u > 0 && player.field_70163_u < 255) {
					if(player.field_71071_by.field_70460_b[2] != null && player.field_71071_by.field_70460_b[2].func_77973_b() == Materials.aquarium.chestplate
							&& player.field_71071_by.field_70460_b[1] != null && player.field_71071_by.field_70460_b[1].func_77973_b() == Materials.aquarium.leggings
							&& player.field_71071_by.field_70460_b[0] != null && player.field_71071_by.field_70460_b[0].func_77973_b() == Materials.aquarium.boots) {
						Block b1 = w.func_180495_p(new BlockPos(player.field_70165_t,player.field_70163_u, player.field_70161_v)).func_177230_c();
						Block b2 = w.func_180495_p(new BlockPos(player.field_70165_t,player.field_70163_u + 1, player.field_70161_v)).func_177230_c();
						if(b1 == Blocks.field_150355_j && b2 == Blocks.field_150355_j) {
							final PotionEffect waterBreathing = new PotionEffect(Potion.field_188414_b.func_82594_a(waterBreathingPotionKey), EFFECT_DURATION, 0, false, false);
							player.func_70690_d(waterBreathing);
							final PotionEffect protection = new PotionEffect(Potion.field_188414_b.func_82594_a(waterBuffPotionKey), EFFECT_DURATION, 0, false, false);
							player.func_70690_d(protection);
                        	player.func_184589_d(Potion.field_188414_b.func_82594_a(fatiguePotionKey));
                        	// Achievement
                        	if (Options.enableAchievements) {
                        		player.func_71064_a(Achievements.scubaDiver, 1);
                        	}
						}
					}
				}
			}
		}
	}

	
	/**
	 *
	 * @param material The material to make the armor from
	 * @param slot Type of armor
	 * @return The armor
	 */
	protected static ItemMetalArmor createArmorBase(@Nonnull MetalMaterial material, EntityEquipmentSlot slot) {
		ArmorMaterial amaterial = Materials.getArmorMaterialFor(material);
		if(amaterial == null) {
			// uh-oh
			FMLLog.severe("Failed to load armor material enum for " + material);
			return null;
		}
		return new ItemMetalArmor(material, amaterial, amaterial.ordinal(), slot);
	}

	/**
	 *
	 * @param material The material to make the helmet from
	 * @return The Helmet
	 */
	public static ItemMetalArmor createHelmet(MetalMaterial material) {
		return createArmorBase(material, EntityEquipmentSlot.HEAD);
	}

	/**
	 * 
	 * @param material The material to make the chestplate from
	 * @return The Chestplate
	 */
	public static ItemMetalArmor createChestplate(MetalMaterial material) {
		return createArmorBase(material, EntityEquipmentSlot.CHEST);
	}

	/**
	 * 
	 * @param material The material to make the leggings from
	 * @return The Leggings
	 */
	public static ItemMetalArmor createLeggings(MetalMaterial material) {
		return createArmorBase(material, EntityEquipmentSlot.LEGS);
	}

	/**
	 * 
	 * @param material The material to make the boots from
	 * @return The Boots
	 */
	public static ItemMetalArmor createBoots(MetalMaterial material) {
		return createArmorBase(material, EntityEquipmentSlot.FEET);
	}

	@SideOnly(Side.CLIENT)
	@Override public String getArmorTexture(ItemStack stack, Entity e, EntityEquipmentSlot slot, String layer) {
		return customTexture;
	}

	@Override
	public void func_77624_a(ItemStack stack, EntityPlayer player, List<String> list, boolean b) {
		super.func_77624_a(stack, player, list, b);
		MetalToolEffects.addArmorSpecialPropertiesToolTip(material, list);
	}

	/**
	 * 
	 * @param item The item
	 * @param world The world
	 * @param crafter The crafter
	 */
	public void extraEffectsOnCrafting(final ItemStack item, final World world, final EntityPlayer crafter) {
		//
	}

	/**
     * @return whether this item is repairable in an anvil.
     */
	@Override public boolean func_82789_a(ItemStack srcItemStack, ItemStack repairMaterial) {
		// repair with string or wool
		List<ItemStack> acceptableItems = OreDictionary.getOres(repairOreDictName);
		for(ItemStack i : acceptableItems ){
			if(ItemStack.func_179545_c(i, repairMaterial))
				return true;
		}
		return false;
	}

	@Override
	public void func_77622_d(final ItemStack item, final World world, final EntityPlayer crafter) {
		super.func_77622_d(item, world, crafter);
		extraEffectsOnCrafting(item, world, crafter);
	}

	@Override
	public MetalMaterial getMaterial() {
		return this.material;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public MetalMaterial getMetalMaterial() {
		return this.material;
	}
}
