package com.mcmoddev.lib.items;

import java.util.List;

import com.mcmoddev.basemetals.init.Materials;
import com.mcmoddev.basemetals.items.MetalToolEffects;
import com.mcmoddev.lib.material.IMetalObject;
import com.mcmoddev.lib.material.MetalMaterial;

import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemAxe;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import net.minecraftforge.oredict.OreDictionary;

import net.minecraft.item.Item.ToolMaterial;

/**
 * Axes
 * 
 * @author DrCyano
 *
 */
public class ItemMetalAxe extends ItemAxe implements IMetalObject {

	protected final MetalMaterial material;
	protected final String repairOreDictName;
	protected final boolean regenerates;
	protected static final long REGEN_INTERVAL = 200;

	/**
	 *
	 * @param material The material to make the axe from
	 */
	public ItemMetalAxe(MetalMaterial material) {
		super(Materials.getToolMaterialFor(material));
		this.material = material;
		this.func_77656_e(this.material.getToolDurability());
		this.field_77865_bY = 4F + (2F * this.material.getBaseAttackDamage());
		this.field_185065_c = -3.5F + Math.min(0.5F, 0.05F * this.material.strength);
		this.field_77864_a = this.material.getToolEfficiency();
		this.repairOreDictName = "ingot" + this.material.getCapitalizedName();
		this.regenerates = this.material.regenerates;
	}

	@Override
	public ToolMaterial func_150913_i() {
		return this.field_77862_b;
	}

	@Override
	public int func_77619_b() {
		return this.field_77862_b.func_77995_e();
	}

	@Override
	public String func_77861_e() {
		return this.field_77862_b.toString();
	}

	@Override
	public boolean func_82789_a(final ItemStack intputItem, final ItemStack repairMaterial) {
		final List<ItemStack> acceptableItems = OreDictionary.getOres(this.repairOreDictName);
		for (final ItemStack i : acceptableItems)
			if (ItemStack.func_179545_c(i, repairMaterial))
				return true;
		return false;
	}

	@Override
	public boolean func_77644_a(final ItemStack item, final EntityLivingBase target, final EntityLivingBase attacker) {
		super.func_77644_a(item, target, attacker);
		MetalToolEffects.extraEffectsOnAttack(this.material, item, target, attacker);
		return true;
	}

	@Override
	public void func_77622_d(final ItemStack item, final World world, final EntityPlayer crafter) {
		super.func_77622_d(item, world, crafter);
		MetalToolEffects.extraEffectsOnCrafting(this.material, item, world, crafter);
	}

	@Override
	public void func_77663_a(final ItemStack item, final World world, final Entity player, final int inventoryIndex, final boolean isHeld) {
		if (this.regenerates && !world.field_72995_K && isHeld && (item.func_77952_i() > 0) && ((world.func_82737_E() % REGEN_INTERVAL) == 0))
			item.func_77964_b(item.func_77952_i() - 1);
	}
/*
	public String getMaterialName() {
		return this.material.getName();
	}
*/
	@Override
	public MetalMaterial getMaterial() {
		return this.material;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public MetalMaterial getMetalMaterial() {
		return this.material;
	}

	@Override
	public void func_77624_a(ItemStack stack, EntityPlayer player, List<String> list, boolean b) {
		super.func_77624_a(stack, player, list, b);
		MetalToolEffects.addToolSpecialPropertiesToolTip(this.material, list);
	}
}
