package com.mcmoddev.lib.items;

import java.util.*;

import com.mcmoddev.basemetals.init.Achievements;
import com.mcmoddev.basemetals.init.Materials;
import com.mcmoddev.basemetals.items.MetalToolEffects;
import com.mcmoddev.basemetals.registry.CrusherRecipeRegistry;
import com.mcmoddev.basemetals.registry.recipe.ICrusherRecipe;
import com.mcmoddev.basemetals.util.Config;
import com.mcmoddev.basemetals.util.Config.Options;
import com.mcmoddev.lib.material.IMetalObject;
import com.mcmoddev.lib.material.MetalMaterial;
import com.mcmoddev.lib.util.Oredicts;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.*;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.*;
import net.minecraft.util.*;
import net.minecraft.util.math.*;
import net.minecraft.world.World;
import net.minecraftforge.oredict.OreDictionary;

import net.minecraft.item.Item.ToolMaterial;

/**
 * Crack Hammers
 */
@SuppressWarnings("deprecation")
public class ItemMetalCrackHammer extends ItemTool implements IMetalObject {
	private static final float ATTACK_SPEED = -3.0F;

	private final MetalMaterial material;
	private final Set<String> toolTypes;
	private final String repairOreDictName;
	private final boolean regenerates;
	private static final long REGEN_INTERVAL = 200;

	/**
	 *
	 * @param material The material to make the crackhammer from
	 */
	public ItemMetalCrackHammer(MetalMaterial material) {
		super(1 + Materials.getToolMaterialFor(material).func_78000_c(), ATTACK_SPEED, Materials.getToolMaterialFor(material), new HashSet<Block>());
		this.material = material;
		field_77865_bY = 5F + 2F * this.material.getBaseAttackDamage();
		field_185065_c = -3.5F;
		func_77656_e((int) (0.75 * this.material.getToolDurability()));
		field_77864_a = this.material.getToolEfficiency();
		toolTypes = new HashSet<>();
		toolTypes.add("crackhammer");
		toolTypes.add("pickaxe");
		repairOreDictName = Oredicts.INGOT + this.material.getCapitalizedName();
		this.regenerates = this.material.regenerates;
	}

	@Override
	public float func_150893_a(final ItemStack tool, final IBlockState target) {
		if (isCrushableBlock(target) && func_150897_b(target)) {
			return Math.max(1.0f, 0.5f * this.material.getToolEfficiency());
		}
		return 1.0f;
	}

	@Override
	public boolean func_179218_a(final ItemStack tool, final World world, final IBlockState target, final BlockPos coord, final EntityLivingBase player) {
		if (!world.field_72995_K && this.func_150897_b(target)) {
			IBlockState bs = world.func_180495_p(coord);
			ICrusherRecipe recipe = getCrusherRecipe(bs);
			if (recipe != null) {
				ItemStack output = recipe.getOutput().func_77946_l();
				world.func_175698_g(coord);
				if (output != null) {
					int num = output.field_77994_a;
					output.field_77994_a = 1;
					for (int i = 0; i < num; i++) {
						world.func_72838_d(new EntityItem(world, coord.func_177958_n() + 0.5, coord.func_177956_o() + 0.5, coord.func_177952_p() + 0.5, output.func_77946_l()));
					}
				}
			}
		}
		return super.func_179218_a(tool, world, target, coord, player);
	}

	@Override
	public EnumActionResult func_180614_a(final ItemStack item, final EntityPlayer player, final World w, final BlockPos coord, EnumHand hand, final EnumFacing facing, final float partialX, final float partialY, final float partialZ) {
		if (facing != EnumFacing.UP) {
			return EnumActionResult.PASS;
		}
		List<EntityItem> entities = w.func_72872_a(EntityItem.class, new AxisAlignedBB(coord.func_177958_n(), coord.func_177956_o() + 1, coord.func_177952_p(), coord.func_177958_n() + 1, coord.func_177956_o() + 2, coord.func_177952_p() + 1));
		boolean success = false;
		for (EntityItem target : entities) {
			ItemStack targetItem = target.func_92059_d();
			if (targetItem != null) {
				ICrusherRecipe recipe = CrusherRecipeRegistry.getInstance().getRecipeForInputItem(targetItem);
				if (recipe != null) {
					// hardness check
					if ((Config.Options.enforceHardness) && (targetItem.func_77973_b() instanceof ItemBlock)) {
						Block b = ((ItemBlock) targetItem.func_77973_b()).func_179223_d();
						if (!this.func_150897_b(b.func_176203_a(targetItem.func_77960_j()))) {
							// cannot harvest the block, no crush for you!
							return EnumActionResult.PASS;
						}
					}
					// crush the item (server side only)
					if (!w.field_72995_K) {
						ItemStack output = recipe.getOutput().func_77946_l();
						int count = output.field_77994_a;
						output.field_77994_a = 1;
						double x = target.field_70165_t;
						double y = target.field_70163_u;
						double z = target.field_70161_v;

						targetItem.field_77994_a--;
						if (targetItem.field_77994_a <= 0) {
							w.func_72900_e(target);
						}
						for (int i = 0; i < count; i++) {
							w.func_72838_d(new EntityItem(w, x, y, z, output.func_77946_l()));
						}
						item.func_77972_a(1, player);
					}
					success = true;
					break;
				}
			}
		}
		if (success) {
			w.func_184133_a(player, coord, SoundEvents.field_187581_bW, SoundCategory.BLOCKS, 0.5F, 0.5F + (field_77697_d.nextFloat() * 0.3F));
		}
		return success ? EnumActionResult.SUCCESS : EnumActionResult.PASS;
	}

	protected boolean isCrushableBlock(IBlockState block) {
		return getCrusherRecipe(block) != null;
	}

	protected boolean isCrushableBlock(Block block) {
		return getCrusherRecipe(block) != null;
	}

	protected ICrusherRecipe getCrusherRecipe(Block block) {
		return getCrusherRecipe(block.func_176223_P());
	}

	protected ICrusherRecipe getCrusherRecipe(IBlockState block) {
		if (block == null || Item.func_150898_a(block.func_177230_c()) == null) {
			return null;
		}
		//return CrusherRecipeRegistry.getInstance().getRecipeForInputItem(block);
		return CrusherRecipeRegistry.getInstance().getRecipeForInputItem(new ItemStack(block.func_177230_c(), 1, block.func_177230_c().func_176201_c(block)));
	}

	@Override
	public ToolMaterial func_150913_i() {
		return field_77862_b;
	}

	@Override
	public int func_77619_b() {
		return field_77862_b.func_77995_e();
	}

	@Override
	public String func_77861_e() {
		return field_77862_b.toString();
	}

	@Override
	public boolean func_82789_a(final ItemStack intputItem, final ItemStack repairMaterial) {
		List<ItemStack> acceptableItems = OreDictionary.getOres(repairOreDictName);
		for (ItemStack i : acceptableItems) {
			if (ItemStack.func_179545_c(i, repairMaterial)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public int getHarvestLevel(final ItemStack item, final String typeRequested) {
		if (typeRequested != null && toolTypes.contains(typeRequested)) {
			if (Config.Options.strongHammers) {
				return material.getToolHarvestLevel();
			}
			else {
				return material.getToolHarvestLevel() - 1;
			}
		}
		return -1;
	}

	@Override
	public Set<String> getToolClasses(final ItemStack item) {
		return toolTypes;
	}

	@Override
	public boolean func_77644_a(final ItemStack item, final EntityLivingBase target, final EntityLivingBase attacker) {
		super.func_77644_a(item, target, attacker);
		MetalToolEffects.extraEffectsOnAttack(material, item, target, attacker);
		return true;
	}

	@Override
	public void func_77622_d(final ItemStack item, final World world, final EntityPlayer crafter) {
		super.func_77622_d(item, world, crafter);
		MetalToolEffects.extraEffectsOnCrafting(material, item, world, crafter);
		// achievement
    	if (Options.enableAchievements) {
    		crafter.func_71064_a(Achievements.geologist, 1);
    	}
	}

	@Override
	public void func_77663_a(final ItemStack item, final World world, final Entity player, final int inventoryIndex, final boolean isHeld) {
		if (regenerates && !world.field_72995_K && isHeld && item.func_77952_i() > 0 && world.func_82737_E() % REGEN_INTERVAL == 0) {
			item.func_77964_b(item.func_77952_i() - 1);
		}
	}

	@Override
	public boolean func_150897_b(final IBlockState targetBS) {
		Block target = targetBS.func_177230_c();
		// go to net.minecraftforge.common.ForgeHooks.initTools(); to see all tool type strings
		String toolType = target.getHarvestTool(target.func_176223_P());
		if (toolTypes.contains(toolType) || target.func_149688_o(targetBS) == Material.field_151576_e) {
			// can mine like a Pickaxe
			return this.getHarvestLevel(null, "pickaxe") >= target.getHarvestLevel(target.func_176223_P());
		}
		else if ("shovel".equals(toolType) && target.getHarvestLevel(target.func_176223_P()) <= 0) {
			// can be dug with wooden shovel
			return true;
		}
		// return true if block doesn't need tools
		return target.getHarvestLevel(target.func_176223_P()) == -1;
	}
/*
	public String getMaterialName() {
		return this.material.getName();
	}
*/
	@Override
	public void func_77624_a(ItemStack stack, EntityPlayer player, List<String> list, boolean b) {
		super.func_77624_a(stack, player, list, b);
		MetalToolEffects.addToolSpecialPropertiesToolTip(this.material, list);
	}

	@Override
	public MetalMaterial getMaterial() {
		return this.material;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public MetalMaterial getMetalMaterial() {
		return this.material;
	}
}
