package com.mcmoddev.lib.items;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.mcmoddev.basemetals.init.Materials;
import com.mcmoddev.basemetals.items.MetalToolEffects;
import com.mcmoddev.lib.material.IMetalObject;
import com.mcmoddev.lib.material.MetalMaterial;
import com.mcmoddev.lib.util.Oredicts;

import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemHoe;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import net.minecraftforge.oredict.OreDictionary;

/**
 * Hoes
 * 
 * @author DrCyano
 *
 */
public class ItemMetalHoe extends ItemHoe implements IMetalObject {

	protected final MetalMaterial material;
	protected final Set<String> toolTypes;
	protected final String repairOreDictName;
	protected final boolean regenerates;
	protected static final long REGEN_INTERVAL = 200;

	/**
	 *
	 * @param material The material to make the hoe from
	 */
	public ItemMetalHoe(MetalMaterial material) {
		super(Materials.getToolMaterialFor(material));
		this.material = material;
		this.func_77656_e(this.material.getToolDurability());
		this.toolTypes = new HashSet<>();
		this.toolTypes.add("hoe");
		this.repairOreDictName = Oredicts.INGOT + this.material.getCapitalizedName();
		this.regenerates = this.material.regenerates;
	}

	@Override
	public boolean func_82789_a(final ItemStack intputItem, final ItemStack repairMaterial) {
		final List<ItemStack> acceptableItems = OreDictionary.getOres(this.repairOreDictName);
		for (final ItemStack i : acceptableItems)
			if (ItemStack.func_179545_c(i, repairMaterial))
				return true;
		return false;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public int getHarvestLevel(final ItemStack item, final String typeRequested) {
		if ((typeRequested != null) && this.toolTypes.contains(typeRequested))
			return this.material.getToolHarvestLevel();
		return -1;
	}

	@Override
	public Set<String> getToolClasses(final ItemStack item) {
		return this.toolTypes;
	}

	@Override
	public float func_150893_a(final ItemStack tool, final IBlockState target) {
		if (this.canHarvestBlock(target, tool))
			return Math.max(1.0f, this.material.getToolEfficiency());
		else
			return 1.0f;
	}

	@Override
	public boolean func_77644_a(final ItemStack item, final EntityLivingBase target, final EntityLivingBase attacker) {
		super.func_77644_a(item, target, attacker);
		MetalToolEffects.extraEffectsOnAttack(this.material, item, target, attacker);
		return true;
	}

	@Override
	public boolean func_150897_b(final IBlockState target) {
		if (this.toolTypes.contains(target.func_177230_c().getHarvestTool(target)))
			return this.material.getToolHarvestLevel() >= target.func_177230_c().getHarvestLevel(target);
		return false;
	}

	@Override
	public void func_77622_d(final ItemStack item, final World world, final EntityPlayer crafter) {
		super.func_77622_d(item, world, crafter);
		MetalToolEffects.extraEffectsOnCrafting(this.material, item, world, crafter);
	}

	@Override
	public void func_77663_a(final ItemStack item, final World world, final Entity player, final int inventoryIndex, final boolean isHeld) {
		if (this.regenerates && !world.field_72995_K && isHeld && (item.func_77952_i() > 0) && ((world.func_82737_E() % REGEN_INTERVAL) == 0))
			item.func_77964_b(item.func_77952_i() - 1);
	}

	@Override
	public MetalMaterial getMaterial() {
		return this.material;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public MetalMaterial getMetalMaterial() {
		return this.material;
	}

	@Override
	public String func_77842_f() {
		return this.material.getName();
	}
}
