package com.mcmoddev.lib.items;

import java.util.List;

import com.mcmoddev.basemetals.init.Materials;
import com.mcmoddev.basemetals.items.MetalToolEffects;
import com.mcmoddev.lib.material.IMetalObject;
import com.mcmoddev.lib.material.MetalMaterial;
import com.mcmoddev.lib.util.Oredicts;

import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.item.ItemSword;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.oredict.OreDictionary;

/**
 * Swords
 * 
 * @author DrCyano
 *
 */
public class ItemMetalSword extends ItemSword implements IMetalObject {
	protected final MetalMaterial material;
	protected final String repairOreDictName;
	protected final boolean regenerates;
	protected static final long REGEN_INTERVAL = 200;
	protected final float attackDamage;

	/**
	 *
	 * @param material The material to make the sword from
	 */
	public ItemMetalSword(MetalMaterial material) {
		super(Materials.getToolMaterialFor(material));
		this.material = material;
		this.func_77656_e(this.material.getToolDurability());
		// this.damageVsEntity = attackDamage + metal.getBaseAttackDamage(); // damageVsEntity is private, sadly
		this.attackDamage = 3F + this.material.getBaseAttackDamage();
		this.repairOreDictName = Oredicts.INGOT + this.material.getCapitalizedName();
		this.regenerates = this.material.regenerates;
		this.func_77637_a(CreativeTabs.field_78037_j);
	}

	@Override
	public boolean func_77644_a(final ItemStack item, final EntityLivingBase target, final EntityLivingBase attacker) {
		item.func_77972_a(1, attacker);
		MetalToolEffects.extraEffectsOnAttack(this.material, item, target, attacker);
		return true;
	}

	@Override
	public boolean func_179218_a(final ItemStack item, final World world, final IBlockState block, final BlockPos coord,
									final EntityLivingBase entity) {
		if (block.func_185887_b(world, coord) != 0.0)
			item.func_77972_a(2, entity);
		return true;
	}

	@Override
	public int func_77619_b() {
		return this.material.getEnchantability();
	}

	@Override
	public boolean func_82789_a(final ItemStack intputItem, final ItemStack repairMaterial) {
		final List<ItemStack> acceptableItems = OreDictionary.getOres(this.repairOreDictName);
		for (final ItemStack i : acceptableItems)
			if (ItemStack.func_179545_c(i, repairMaterial))
				return true;
		return false;
	}

	@Override
	public void func_77622_d(final ItemStack item, final World world, final EntityPlayer crafter) {
		super.func_77622_d(item, world, crafter);
		MetalToolEffects.extraEffectsOnCrafting(this.material, item, world, crafter);
	}

	@Override
	public void func_77663_a(final ItemStack item, final World world, final Entity player, final int inventoryIndex, final boolean isHeld) {
		if (this.regenerates && !world.field_72995_K && isHeld && (item.func_77952_i() > 0) && ((world.func_82737_E() % REGEN_INTERVAL) == 0))
			item.func_77964_b(item.func_77952_i() - 1);
	}

	/**
	 *
	 * @return The amount of damage dealt to an entity when attacked by this
	 * item
	 */
	public float getAttackDamage() {
		return this.attackDamage;
	}
/*
	public String getMaterialName() {
		return this.material.getName();
	}
*/
	@Override
	public void func_77624_a(ItemStack stack, EntityPlayer player, List<String> list, boolean b) {
		super.func_77624_a(stack, player, list, b);
		MetalToolEffects.addToolSpecialPropertiesToolTip(this.material, list);
	}

	@Override
	public MetalMaterial getMaterial() {
		return this.material;
	}

	/**
	 * @deprecated
	 */
	@Override
	@Deprecated
	public MetalMaterial getMetalMaterial() {
		return this.material;
	}
}
